@@ -8,15 +8,28 @@ __copyright__ = "(C) 2010 Science and Technology Facilities Council"
 __license__ = "http://www.apache.org/licenses/LICENSE-2.0"
 __contact__ = "Philip.Kershaw@stfc.ac.uk"
 __revision__ = '$Id: etree.py 7131 2010-06-30 13:37:48Z pjkersha $'
-try: # python 2.5
-    from xml.etree import ElementTree
-except ImportError:
-    # if you've installed it yourself it comes this way
-    import ElementTree
+from ndg.saml import Config, importElementTree
+ElementTree = importElementTree()
 
 import re
 
 
+if Config.use_lxml:
+    def makeEtreeElement(tag, ns_prefix, ns_uri, attrib={}, **extra):
+        """Makes an ElementTree element handling namespaces in the way
+        appropriate for the ElementTree implementation in use.
+        """
+        elem = ElementTree.Element(tag, {ns_prefix: ns_uri}, attrib, **extra)
+        return elem
+else:
+    def makeEtreeElement(tag, ns_prefix, ns_uri, attrib={}, **extra):
+        """Makes an ElementTree element handling namespaces in the way
+         appropriate for the ElementTree implementation in use.
+        """
+        elem = ElementTree.Element(tag, attrib, **extra)
+        ElementTree._namespace_map[ns_uri] = ns_prefix
+        return elem
+
 class QName(ElementTree.QName):
     """XML Qualified Name for ElementTree
     
@@ -100,23 +113,53 @@ class QName(ElementTree.QName):
 
 
 def prettyPrint(*arg, **kw):
-    '''Lightweight pretty printing of ElementTree elements'''
+    '''Lightweight pretty printing of ElementTree elements.  This function
+    wraps the PrettyPrint class
+    
+    @param arg: arguments to pretty print function
+    @type arg: tuple
+    @param kw: keyword arguments to pretty print function
+    @type kw: dict
+    '''
     
     # Keep track of namespace declarations made so they're not repeated
     declaredNss = []
-    
-    _prettyPrint = _PrettyPrint(declaredNss)
-    return _prettyPrint(*arg, **kw)
+    if not Config.use_lxml:
+        mappedPrefixes = dict.fromkeys(ElementTree._namespace_map.values(), True)
+        namespace_map_backup = ElementTree._namespace_map.copy()
+    else:
+        mappedPrefixes = {}
+
+    _prettyPrint = _PrettyPrint(declaredNss, mappedPrefixes)
+    result = _prettyPrint(*arg, **kw)
+
+    if not Config.use_lxml:
+        ElementTree._namespace_map = namespace_map_backup
+
+    return result
 
 
 class _PrettyPrint(object):
+    '''Class for lightweight pretty printing of ElementTree elements'''
     MAX_NS_TRIES = 256
-    def __init__(self, declaredNss):
+    def __init__(self, declaredNss, mappedPrefixes):
+        """
+        @param declaredNss: declared namespaces
+        @type declaredNss: iterable of string elements
+        @param mappedPrefixes: map of namespace URIs to prefixes
+        @type mappedPrefixes: map of string to string
+        """
         self.declaredNss = declaredNss
+        self.mappedPrefixes = mappedPrefixes
     
     @staticmethod
     def estrip(elem):
-        ''' Just want to get rid of unwanted whitespace '''
+        '''Utility to remove unwanted leading and trailing whitespace 
+        
+        @param elem: ElementTree element
+        @type elem: ElementTree.Element
+        @return: element content with whitespace removed
+        @rtype: basestring'''
         if elem is None:
             return ''
         else:
@@ -124,39 +167,27 @@ class _PrettyPrint(object):
             # wrapper it as a string
             return str(elem).strip()
         
-    def _allocNsPrefix(self, nsURI):
-        """Allocate a namespace prefix if one is not already set for the given
-        Namespace URI
-        """
-        nsPrefix = ElementTree._namespace_map.get(nsURI)
-        if nsPrefix is not None:
-            return nsPrefix
-        
-        for i in range(self.__class__.MAX_NS_TRIES):
-            nsPrefix = "ns%d" % i
-            if nsPrefix not in self.declaredNss:
-                ElementTree._namespace_map[nsURI] = nsPrefix
-                break
-            
-        if nsURI not in ElementTree._namespace_map:                            
-            raise KeyError('prettyPrint: error adding namespace '
-                           '"%s" to ElementTree._namespace_map' % 
-                           nsURI)   
-        
-        return nsPrefix
-                 
     def __call__(self, elem, indent='', html=0, space=' '*4):
         '''Most of the work done in this wrapped function - wrapped so that
         state can be maintained for declared namespace declarations during
-        recursive calls using "declaredNss" above'''  
+        recursive calls using "declaredNss" above
+        
+        @param elem: ElementTree element
+        @type elem: ElementTree.Element
+        @param indent: set indent for output
+        @type indent: basestring
+        @param space: set output spacing
+        @type space: basestring 
+        @return: pretty print format for doc
+        @rtype: basestring       
+        '''  
         strAttribs = []
         for attr, attrVal in elem.attrib.items():
             nsDeclaration = ''
             
             attrNamespace = QName.getNs(attr)
             if attrNamespace:
-                # Allocate a prefix
-                nsPrefix= self._allocNsPrefix(attrNamespace)
+                nsPrefix = self._getNamespacePrefix(elem, attrNamespace)
                 
                 attr = "%s:%s" % (nsPrefix, QName.getLocalPart(attr))
                 
@@ -169,7 +200,7 @@ class _PrettyPrint(object):
         strAttrib = ''.join(strAttribs)
         
         namespace = QName.getNs(elem.tag)
-        nsPrefix = self._allocNsPrefix(namespace)
+        nsPrefix = self._getNamespacePrefix(elem, namespace)
             
         tag = "%s:%s" % (nsPrefix, QName.getLocalPart(elem.tag))
         
@@ -189,7 +220,7 @@ class _PrettyPrint(object):
         if children:
             for child in elem:
                 declaredNss = self.declaredNss[:]
-                _prettyPrint = _PrettyPrint(declaredNss)
+                _prettyPrint = _PrettyPrint(declaredNss, self.mappedPrefixes)
                 result += '\n'+ _prettyPrint(child, indent=indent+space) 
                 
             result += '\n%s%s</%s>' % (indent,
@@ -200,3 +231,39 @@ class _PrettyPrint(object):
             
         return result
 
+    if Config.use_lxml:
+        def _getNamespacePrefix(self, elem, namespace):
+            for nsPrefix, ns in elem.nsmap.iteritems():
+                if ns == namespace:
+                    return nsPrefix
+            raise KeyError('prettyPrint: missing namespace "%s" for '
+                               'elem.nsmap' % namespace)
+    else:
+        def _getNamespacePrefix(self, elem, namespace):
+            nsPrefix = self._allocNsPrefix(namespace)
+            if nsPrefix is None:
+                raise KeyError('prettyPrint: missing namespace "%s" for '
+                               'ElementTree._namespace_map' % namespace)
+            return nsPrefix
+
+        def _allocNsPrefix(self, nsURI):
+            """Allocate a namespace prefix if one is not already set for the given
+            Namespace URI
+            """
+            nsPrefix = ElementTree._namespace_map.get(nsURI)
+            if nsPrefix is not None:
+                return nsPrefix
+
+            for i in range(self.__class__.MAX_NS_TRIES):
+                nsPrefix = "ns%d" % i
+                if nsPrefix not in self.mappedPrefixes:
+                    ElementTree._namespace_map[nsURI] = nsPrefix
+                    self.mappedPrefixes[nsPrefix] = True
+                    break
+
+            if nsURI not in ElementTree._namespace_map:                            
+                raise KeyError('prettyPrint: error adding namespace '
+                               '"%s" to ElementTree._namespace_map' % 
+                               nsURI)   
+
+            return nsPrefix
